=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2009-01-08 16:57:10 +0000
+++ bzrlib/repofmt/pack_repo.py	2009-02-19 03:37:40 +0000
@@ -1971,6 +1971,13 @@
         self._pack_collection._start_write_group()
 
     def _commit_write_group(self):
+        vfs = [self.revisions, self.inventories, self.texts, self.signatures]
+        for vf in vfs:
+            missing = vf._index.get_missing_compression_parents()
+            if missing:
+                raise errors.BzrCheckError(
+                    "Repository %s has missing compression parent(s) %r "
+                    "in %r" % (self, list(missing), vf))
         return self._pack_collection._commit_write_group()
 
     def get_transaction(self):

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2009-02-18 05:42:35 +0000
+++ bzrlib/tests/test_knit.py	2009-02-19 03:37:40 +0000
@@ -1628,6 +1628,9 @@
         combined = CombinedGraphIndex([unvalidated])
         index = _KnitGraphIndex(combined, lambda: True)
         index._scan_unvalidated_index(unvalidated)
+        # This also checks that its only the compression parent that is
+        # examined, otherwise 'ghost' would also be reported as a missing
+        # parent.
         self.assertEqual(
             frozenset([('missing-parent',)]),
             index.get_missing_compression_parents())
@@ -1669,7 +1672,6 @@
               ([('parent-one',)], [('parent-one',)]))])
         combined = CombinedGraphIndex([graph_index_a, graph_index_b])
         index = _KnitGraphIndex(combined, lambda: True)
-
         index._scan_unvalidated_index(graph_index_a)
         index._scan_unvalidated_index(graph_index_b)
         self.assertEqual(

=== modified file 'bzrlib/tests/test_pack_repository.py'
--- bzrlib/tests/test_pack_repository.py	2008-11-27 07:25:52 +0000
+++ bzrlib/tests/test_pack_repository.py	2009-02-19 03:37:40 +0000
@@ -533,6 +533,48 @@
         self.assertRaises(errors.NoSuchRevision,
             missing_ghost.get_inventory, 'ghost')
 
+    def make_write_ready_repo(self):
+        repo = self.make_repository('.', format=self.get_format())
+        repo.lock_write()
+        repo.start_write_group()
+        return repo
+
+    def test_missing_inventories_compression_parent_prevents_commit(self):
+        repo = self.make_write_ready_repo()
+        key = ('junk',)
+        repo.inventories._index._missing_compression_parents.add(key)
+        self.assertRaises(errors.BzrCheckError, repo.commit_write_group)
+        self.assertRaises(errors.BzrCheckError, repo.commit_write_group)
+        repo.abort_write_group()
+        repo.unlock()
+
+    def test_missing_revisions_compression_parent_prevents_commit(self):
+        repo = self.make_write_ready_repo()
+        key = ('junk',)
+        repo.revisions._index._missing_compression_parents.add(key)
+        self.assertRaises(errors.BzrCheckError, repo.commit_write_group)
+        self.assertRaises(errors.BzrCheckError, repo.commit_write_group)
+        repo.abort_write_group()
+        repo.unlock()
+
+    def test_missing_signatures_compression_parent_prevents_commit(self):
+        repo = self.make_write_ready_repo()
+        key = ('junk',)
+        repo.signatures._index._missing_compression_parents.add(key)
+        self.assertRaises(errors.BzrCheckError, repo.commit_write_group)
+        self.assertRaises(errors.BzrCheckError, repo.commit_write_group)
+        repo.abort_write_group()
+        repo.unlock()
+
+    def test_missing_text_compression_parent_prevents_commit(self):
+        repo = self.make_write_ready_repo()
+        key = ('some', 'junk')
+        repo.texts._index._missing_compression_parents.add(key)
+        self.assertRaises(errors.BzrCheckError, repo.commit_write_group)
+        e = self.assertRaises(errors.BzrCheckError, repo.commit_write_group)
+        repo.abort_write_group()
+        repo.unlock()
+
     def test_supports_external_lookups(self):
         repo = self.make_repository('.', format=self.get_format())
         self.assertEqual(self.format_supports_external_lookups,

